class InvalidRequestException(Exception):
    pass


class ConfigParserException(Exception):
    pass


class BlockerException(Exception):
    pass


class SshException(Exception):
    pass


class InvalidParameterException(Exception):
    pass


class LockException(Exception):
    pass


class ModuleException(Exception):
    pass


class RecipeException(Exception):
    pass


class DeployException(Exception):
    pass
