DEPLOY_PENDING = 'pending'
DEPLOY_SUCCESS = 'success'
DEPLOY_FAIL = 'fail'

INVALID_JSON = 'INVALID_JSON'
INVALID_REQUEST = 'INVALID_REQUEST'
INVALID_TOKEN = 'INVALID_TOKEN'
INVALID_RELEASE = 'INVALID_RELEASE'
RELEASE_DEPLOYED = 'RELEASE_DEPLOYED'
READ_ONLY = 'READ_ONLY'

INPUT_BEGIN = 'begin'
INPUT_END = 'end'
INPUT_MESSAGE = 'message'

WEBSOCKET_PREFIX = '/input'
